/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.request;

import org.netkernel.urii.*;

/** All NetKernel resource requests are initiated by an IRequestor client,
 * modeled as an IRequest object and are sent to either an IAsyncRequestee or
 * an ISyncRequestee object.
 * <p>
 * An IRequest object requires a <i>verb</i>
 * (Modelled as a Java int), an <i>identifier</i> (modelled as IIdentifier)
 * and may optionally specify a specific <i>representation class</i> (modeled as
 * any Java class) for the returned resource-representation.
 * <p>
 * If the IRequest is sent to the IAsynchronousRequestee interface it must
 * include a reference to the IRequestor client issuing the request as this
 * interface provides the callback method onResponse().
 * <p>
 * Each request must have a requested execution priority specified by a Java int value.
 * Three preset priorities are PRIORITY_MIN, PRIORITY_STD, and PRIORITY_MAX.
 * <p>
 * The parent request is the request that causes an endpoint or other code
 * to issue a sub-request. The getParent() method returns a reference to
 * the parent request of this request.
 * <p>
 * All requests are evaluated within a specific space. The full scope of the
 * space is available from the getRequestScope() method.
 * <p>
 *
 * @author  tab
 * @see org.netkernel.urii.IIdentifier
 * @see IAsyncRequestee
 * @see ISynchronousRequestee
 * @see IRequestScopeLevel
 * @see IRequestor
 * 
 */
public interface IRequest
{
	// The verbs
	
	/** verb specifying the return of a resource-representation */
	public static final int VERB_SOURCE=1;
	/** verb specifying that a resource is to be set to the supplied resource-representation */
	public static final int VERB_SINK=2;
	/** verb specifying a test for existence of a resource */
	public static final int VERB_EXISTS=4;
	/** verb specifying the deletion of a resource */
	public static final int VERB_DELETE=8;
	/** verb specifying the creation of a new resource */
	public static final int VERB_NEW=16;
	/** verb specifying the transreption of a resource-representation to another class type */
	public static final int VERB_TRANSREPT=32;
	/** verb specifying the fragmentation of a representation */
	public static final int VERB_FRAGMENT=64;
	/** verb specifying the resolution of the identifier */
	public static final int VERB_RESOLVE=128;
	/** verb specifying meta-data level information related to a request or endpoint */
	public static final int VERB_META=256;
	
	public static final int VERB_ALL=VERB_META|VERB_TRANSREPT|VERB_NEW|VERB_DELETE|VERB_EXISTS|VERB_SINK|VERB_SOURCE;

	// Preset priority values
	
	/** The lowest execution priority value */
	public static final int PRIORITY_MIN=0;
	/** The default execution priority value */
	public static final int PRIORITY_STD=1;
	/** The highest execution priority value */
	public static final int PRIORITY_MAX=2;	
	
	/** Header to declare that this request should not have its dependencies tracked because it is likely
	 * to have too many and that may cause the system to run out of memory tracking them. By not tracking
	 * dependencies a response will always be expired and will have an unexpectedly low cost.
	 * Existence of header causes dependencies to be forgot.
	 */
	public static final String HEADER_FORGET_DEPENDENCIES="forget-dependencies";	
	
	public static final String HEADER_PRIORITY="priority";
	
	/** Header to declare that this request should not have it's response cached and that it's response
	 * should not be retrieved from cache. Usually it is for the endpoint providing
	 * the response to determine if the representation should be cached. In some rare situations it is useful for the client
	 * to force no caching. 
	 * Existence of header inhibits caching.
	 */
	public static final String HEADER_NO_CACHE="no-cache";

	/** Returns the resource identifier specified for this request.
   *
   * This value is never null.
   *
	 * @return Request resource identifier.
	 */
	IIdentifier getIdentifier();
	
	/** Returns the verb specified for this request.
	 *
   * @return Request verb.
	 */
	int getVerb();
	
	/** Returns the innermost or closest space in the request scope stack.
	 *
   * @return Request scope
	 */
	IRequestScopeLevel getRequestScope();
	
	/** Returns the Java class specified in this request. This may be a null
   * value for some requests.
   * <p>
   * Requests issued to the kernel and requests for transreption
   * (VERB_TRANSREPT)
   * must have a non-null representation class. Requests to
   * an endpoint may have a null value.
   *
	 * @return Resource-representation class.
	 */
	@SuppressWarnings("unchecked")
	Class getRepresentationClass();
	
	/** Returns the execution priority for this request.
	 * <p>
   *
   * @return actual execution priority.
	 */
	int getPriority();
	
	/** Return the parent request of this request.
   * <p>
   * A parent request is the request that was issued to an endpoint that
   * in turn issued this request.
   *
   * A null value is returned if and only if this is an initial request initiated in
   * the root space by a transport.
	 *
   * @return The request that invoked the endpoint that caused the current request
   * to be executed.
	 */
	IRequest getParent();
	
	/** Returns the IRequestor object that initiated this request
   * and which is to receive the asynchronous callback.
	 *
   * @return Requestor that issued this request and which is to receive the asynchronous callback.
	 */
	IRequestor getRequestor();
	
	/** Return request headers
	 * @return
	 */
	IRequestResponseFields getHeaders();
	
	// values for resolve request
	public static final int TYPE_RESOLVE_REQUEST=0;
	public static final int TYPE_RESOLVE_EXCLUDED_SPACES=1;
	public static final int TYPE_RESOLVE_DEBUG=2;

	//values for transrept, sink, new
	public static final int TYPE_SOURCED_RESPONSE=0;
	
	//values for meta
	public static final int TYPE_QUALIFYING_REQUEST=0;
	
	//value for all non-resolve requests
	public static final int TYPE_RESOLVED_ENDPOINT_IDENTIFIER=1;
	
	/** Get an implementation optimisation value from the request. These values are used for very specific purposes
	 * on requests with certain verbs as defined above. Advanced use -not for general purpose use.
     *
	 * @param aType one of the TYPE_ constants in this interface
	 * @return
	 */ 
	Object getValue(int aType);
	Object[] getValues();
}