/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.scheduler;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

import org.netkernel.container.IKernel;

/** A ProcessofThreadGroup is a homogeneous collection of ProcessorThreads which can
 * be managed as a group. This includes start/stop/suspend/resume. But also the
 * groups size can be change dynamically.
 */
public class WorkerThreadPool
{
    /** The real java ThreadGroup that threads in this ProcessorThreadGroup belong.
     */
    private ThreadGroup mThreadGroup;
    /** The set of active threads in this group
     */
    private final ArrayList<WorkerThread> mProcessorThreads = new ArrayList<WorkerThread>();
    /** The set of dying or dead threads that this group has finished with
     */
    private final ArrayList<WorkerThread> mDyingThreads = new ArrayList<WorkerThread>();
	/** reference to scheduler
	 */
	private Scheduler mScheduler;
	/** last set thread count 
	 */
	private int mDesiredThreadCount;
	
	private AtomicInteger mThreadId = new AtomicInteger();
	
    /** state of scheduler 
     */
    private int mState;
	public static final int STATE_STOPPED=0;
	public static final int STATE_STARTED=1;
	public static final int STATE_STOPPING=2;
    
    /** Creates a new ProcessorThreadGroup.
     * @param aName The name that the real java ThreadGroup will be given
     * @param aInitialThreads The initial number of threads that this group will have.
	 * @param aParentGroup parent thread group
     */
    public WorkerThreadPool(String aName, int aInitialThreads, ThreadGroup aParentGroup, Scheduler aScheduler)
    {	mThreadGroup = new ThreadGroup(aParentGroup, aName);
    	mScheduler=aScheduler;
		mDesiredThreadCount=aInitialThreads;
		mState=STATE_STOPPED;
    }
    
    /** Returns the real java ThreadGroup object for this group.
     * @return The java.lang.ThreadGroup.
     */
    public ThreadGroup getThreadGroup()
    {   return mThreadGroup;
    }
    
    /** Dynamically changes the number of active threads within this group. It is
     * valid to call this method at any time before the group is stopped.
     * @param aThreadCount The desired number of active threads for the group.
     */
    public void setCount(int aThreadCount)
    {  mDesiredThreadCount=aThreadCount;
		if (mState==STATE_STARTED)
		{	int actualCount = threadCount();
			if (actualCount<aThreadCount)
			{   int threadsToCreate = aThreadCount - actualCount;
				for (int i=0; i<threadsToCreate; i++)
				{   WorkerThread pt = createNewThread(mThreadGroup);
					mProcessorThreads.add(pt);
					pt.start();
				}
			}
			else if (actualCount>aThreadCount)
			{   int threadsToStop = actualCount - aThreadCount;
				Iterator<WorkerThread> iterator = mProcessorThreads.iterator();
				// stopped threads are moved onto mDyingThreads so
				// that they can be tracked until dead.
				for (int i=0; i<threadsToStop; i++)
				{   WorkerThread pt = iterator.next();
					pt.requestStop();
					iterator.remove();
					mDyingThreads.add(pt);
				}
			}
		}
		
		
		/*
		 * Experimental thread profiling code
		 * To try to determine worker thread affinity to CPU cores
		if (actualCount<aThreadCount)
		{	//System.out.println("PROFILE "+aThreadCount);
			ProfileThreadState[] pts = new ProfileThreadState[aThreadCount];
			for (int i=0; i<aThreadCount; i++)
			{	pts[i]=new ProfileThreadState();
				mScheduler.getRequestTable().put(pts[i]);
			}
			
			try
			{	//compare each worker thread against each other
			for (int a=0; a<aThreadCount; a++)
			{	for (int b=0; b<a; b++)
				{		
						long t0=System.currentTimeMillis();
						pts[a].work();
						pts[b].work();
						pts[a].waitForWork();
						pts[b].waitForWork();
						long t=System.currentTimeMillis()-t0;
						System.out.println("PROFILE "+a+" "+b+" ->"+t);
				}
			}
			}
			catch (InterruptedException e)
			{
			}
			
			for (int i=0; i<aThreadCount; i++)
			{	pts[i].release();
			}
		}
		*/
    }
    
    public int getCount()
    {	return mDesiredThreadCount;
    }
    
    /** The count of the number of active threads in this group.
     * @return The count of the number of active threads in this group.
     */
    public int threadCount()
    {   purgeDyingThreads();
        return mProcessorThreads.size()+mDyingThreads.size();
    }
    
    public List<WorkerThread> getActiveThreads()
    {	return mProcessorThreads;
    }
    
    /** Internal housekeeping method to forget about threads which have now been
     * confirmed dead.
     */
    private void purgeDyingThreads()
    {   for (Iterator<WorkerThread> i = mDyingThreads.iterator(); i.hasNext(); )
        {   WorkerThread pt = i.next();
            if (!pt.isAlive())
            {   i.remove();
            }
        }
    }
    
    /** Abstract method to instantiate a new instance of a subclass of ProcessorThread
     * to be added to this group.
     * @param aThisGroup The ThreadGroup that the new thread should belong to.
     * @return The newly created ProcessorThread.
     */
    protected WorkerThread createNewThread(ThreadGroup aThisGroup)
	{	return new WorkerThread(this,aThisGroup);
	}
    
    /** Starts all the threads in the group.
     */
    public void start()
    {	if (getState()!=STATE_STARTED)
		{	mState=STATE_STARTED;
			mThreadId.set(0);
			setCount(mDesiredThreadCount);
        }
    }
    
    /** Requests that all the threads in the group are stopped.
     * @param aForce if false no threads are harmed that cannot be stopped naturally. If true then threads are forced to stop
     */
    public void stop(boolean aForce)
	{  if (aForce)
	   {	for (Iterator<WorkerThread> i = mProcessorThreads.iterator(); i.hasNext(); )
			{	WorkerThread pt = i.next();
				pt.forceStop();
				i.remove();
			}
			for (Iterator<WorkerThread> i = mDyingThreads.iterator(); i.hasNext(); )
			{	WorkerThread pt = i.next();
				pt.forceStop();
				i.remove();
			}
			mState=STATE_STOPPED;
	   }
	   else
	   {	if (mState==STATE_STARTED)
			{	setCount(0);
				mState=STATE_STOPPING;
			}
	   }
    }
	
	public int getState()
	{	return mState;
	}

	public IKernel getKernel()
	{	return mScheduler.getKernel();
	}
	
	public String getThreadId()
	{	return Integer.toString(mThreadId.getAndIncrement());
	}
	
	void process() throws InterruptedException
	{
		IRequestState state;
		try	
		{	state = mScheduler.getRequestTable().take();
		} catch (InterruptedException e)
		{	state = null;
			throw e;
		}
		if (state!=null)
		{	state.processRequest();
		}
	}
}